#!/bin/bash
# Creates the proper directories, users and permissions for SQL Server extensibility.

echo "Running `basename ${0}`..."
# Create all the groups
if ! getent group mssql_launchpadd >/dev/null; then
    groupadd -r mssql_launchpadd
fi

if ! getent group mssql_extmgr >/dev/null; then
    groupadd -r mssql_extmgr
fi

if ! getent group mssql_satellite >/dev/null; then
    groupadd -r mssql_satellite
fi
echo "mssql-extensibility groups have been created."

# Create required users
# The script assumes mssql user exists
# Add mssql user to mssql_launchpadd and mssql_extmgr groups.
usermod -a -G mssql_launchpadd,mssql_extmgr mssql

if ! getent passwd mssql_launchpadd >/dev/null; then
    useradd -M -r -s /bin/bash -g mssql_launchpadd mssql_launchpadd
else
    usermod -g mssql_launchpadd mssql_launchpadd
fi

if ! getent passwd mssql_extmgr >/dev/null; then
    useradd -M -r -s /bin/bash -g mssql_extmgr mssql_extmgr
else
    usermod -g mssql_extmgr mssql_extmgr
fi

if ! getent passwd mssql_satellite >/dev/null; then
    useradd -M -r -s /bin/bash -g mssql_satellite mssql_satellite
else
    usermod -g mssql_satellite mssql_satellite
fi
echo "mssql-extensibility users have been created."

# Create folders
MSSQL_EXT_DIR="/var/opt/mssql-extensibility"
MSSQL_EXT_DATA_DIR="${MSSQL_EXT_DIR}/data"
if [ ! -d "${MSSQL_EXT_DATA_DIR}" ]; then
    mkdir -p ${MSSQL_EXT_DATA_DIR}
fi

MSSQL_EXT_LOG_DIR="${MSSQL_EXT_DIR}/log"
if [ ! -d "${MSSQL_EXT_LOG_DIR}" ]; then
    mkdir -p ${MSSQL_EXT_LOG_DIR}
fi

MSSQL_EXT_LIB_TEMP_DIR="${MSSQL_EXT_DIR}/externallibrariestemp"
if [ ! -d "${MSSQL_EXT_LIB_TEMP_DIR}" ]; then
    mkdir -p ${MSSQL_EXT_LIB_TEMP_DIR}
fi

MSSQL_EXT_LIB_DIR="${MSSQL_EXT_DIR}/externallibraries"
if [ ! -d ${MSSQL_EXT_LIB_DIR} ]; then
    mkdir -p ${MSSQL_EXT_LIB_DIR}
fi

MSSQL_EXT_LANG_TEMP_DIR="${MSSQL_EXT_DIR}/externallanguagestemp"
if [ ! -d "${MSSQL_EXT_LANG_TEMP_DIR}" ]; then
    mkdir -p ${MSSQL_EXT_LANG_TEMP_DIR}
fi

MSSQL_EXT_LANG_DIR="${MSSQL_EXT_DIR}/externallanguages"
if [ ! -d ${MSSQL_EXT_LANG_DIR} ]; then
    mkdir -p ${MSSQL_EXT_LANG_DIR}
fi

SATELLITE_HOME_DIR="/home/mssql_satellite"
if [ ! -d "${SATELLITE_HOME_DIR}" ]; then
    mkdir -p ${SATELLITE_HOME_DIR}
fi

MSSQL_EXT_SANDBOX_DIR="${MSSQL_EXT_DIR}/sandboxes"
if [ ! -d ${MSSQL_EXT_SANDBOX_DIR} ]; then
    mkdir -p ${MSSQL_EXT_SANDBOX_DIR}
fi

chown mssql_launchpadd:mssql_launchpadd ${MSSQL_EXT_DIR} ${MSSQL_EXT_DIR}/*
chmod 771 ${MSSQL_EXT_DIR} ${MSSQL_EXT_DIR}/*
# set group inheritance bit for external languages and libraries directories
chmod g+s ${MSSQL_EXT_LANG_TEMP_DIR} ${MSSQL_EXT_LIB_TEMP_DIR} ${MSSQL_EXT_LANG_DIR} ${MSSQL_EXT_LIB_DIR}

chown mssql_launchpadd:mssql_launchpadd $SATELLITE_HOME_DIR
chmod 771 $SATELLITE_HOME_DIR

echo "mssql-extensibility directories have been created."

# Set capabilities
# CAP_CHOWN: file permissions.
# CAP_DAC_OVERRIDE: create cgroups.
# CAP_SETGID, CAP_SETUID: run script as low privileged user.
# CAP_SYS_ADMIN: create namespaces.
# CAP_KILL: kill launchpad
chown mssql_launchpadd:mssql_launchpadd /opt/mssql/bin/launchpadd
chmod 750 /opt/mssql/bin/launchpadd
setcap cap_chown,cap_dac_override,cap_setuid,cap_setgid,cap_kill,cap_sys_admin,cap_mknod+ep /opt/mssql/bin/launchpadd

# CAP_DAC_OVERRIDE, CAP_NET_RAW, CAP_SETPCAP: iptables.
# CAP_SYS_PTRACE: enter network namespace of another process.
# CAP_NET_ADMIN, CAP_SYS_ADMIN: manage network interfaces.
chown mssql_launchpadd:mssql_launchpadd /opt/mssql/bin/setnetbr
chmod 750 /opt/mssql/bin/setnetbr
setcap cap_net_admin,cap_net_raw,cap_sys_ptrace,cap_sys_admin,cap_dac_override,cap_setpcap+ep /opt/mssql/bin/setnetbr
echo "Fine grained capabilities have been granted to launchpadd."

echo "`basename ${0}` done."
